######################################################################
#:CQCC::IOPtkPanel
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2001, 2008 All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#

=head1 NAME 

CQCC::IOPtkPanel - IOPtk widget class for managing panel widgets

=head1 CLASS DESCRIPTION

This class provides wrapper objects for a number of commonly used
geometry management widgets, hiding the implementation details and
handling message translation and child management. 

As a subclass of IOPtkWidget, it inherits some basic attribute 
management and initialization functionality. In addition this class
manages zero or more child IOPtkWidget objects including other panels.

How the IOPtkPanel object looks and behaves depends on its "subtype"
which provides a lightweight subclassing feature. Each subtype has an 
initialization function that uses Perl/TK calls to define the widget;
the subtype may have additional supporting methods to implement its
behavior. 

The currently supported subtypes include IOMainWindow, IOTopLevel,
IODialogBox, IOFrame. See the method documentation on each for more
information.

The IOPtkPanel ClassTest method provides an extensive test of all 
of the IOPtkWidget classes. 

SUPPORT POLICY: See TriggerCQCC.pm for the Rational Support Policy. 

=head1 METHODS

=cut

######################################################################
#::Package declarations
######################################################################

require 5.006;
require Config;

package CQCC::IOPtkPanel;
use Exporter;
use CQCC::IOPtkWidget;
@ISA = qw( CQCC::IOPtkWidget );

use Tk 800.022;
use Tk::DialogBox;
use strict;

use CQCC::Parm;
use CQCC::MsgCat;
use CQCC::IOBase;
use CQCC::IOPerlTK;
use CQCC::IOPtkManip;

sub Version { return '2.1' };

######################################################################
#::Class Globals 
######################################################################

######################################################################
#::function ClassTest

=over 6

=item C<ClassTest(@args)>

This class function runs an internal class test which creates
a Notebook with two Notebook pages, each containing instances of the 
IOPtkPanel, IOPtkMenu, and IOPtkManip classes to exercise and test
behavior. 

=back

=cut

######################################################################
my $IO;

sub ClassTest { 
    my @args = @_;
    &ForceParm("IOBaseInterface", "CQCC::IOPerlTK");

    $IO = &IOstd();

    $IO->Panel("Main", "", "IOMainWindow");

    $IO->Panel("Frame1", "Main", "IOFrame",
               TITLE=>"Frame1",
               PACK=>[-side => "left", -fill => 'y']);

    $IO->Manipulator("Label1", "Frame1", "IOLabel",
               TITLE=>"Label1", PARM=>"CQCC_CTRL_C");

    $IO->Manipulator("List1", "Frame1", "IOListbox",
               TITLE=>"List", 
               RANGE => ["a","b","c"],
               PACK=>[-side => "left", -fill => 'both']);

    my @data = ( [ "ID", "Heading", "State" ],
                 [ "SAMPL0001", "Test heading 1", "Open" ],
                 [ "SAMPL0021", "Test heading 2", "Assigned" ],
                 [ "SAMPL0034", "Test heading 3", "Closed" ],
                 [ "SAMPL0056", "Test heading 4", "Submit" ]);

    $IO->Manipulator("ListboxGrid", "Frame1", "IOListboxGrid",
               TITLE=>"ListboxGrid", 
               RANGE => \@data,
               PACK=>[-side => "left", -fill => 'both']);

    my @q = ("Work/Personal Queries/WebQueries/Report 0",
             "Work/Public Queries/Aging Charts/Closed 30 Day report",
             "Work/Public Queries/Aging Charts/Aging by State report",
             "Work/Public Queries/Aging Charts/Open/7 Day report",
             "Work/Public Queries/Aging Charts/Open/30 Day report",
            );

    $IO->Manipulator("List2", "Frame1", "IOListboxTree",
               TITLE=>"Tree", 
               RANGE => \@q,
               PACK=>[-side => "left", -fill => 'both']);

    $IO->Menu("Menu1", "Frame1", "IOCommandMenu", 
               TITLE=>"Buttons", 
               MENULIST => 
                   [ "OK "          => 'print "OK\n";',
                     "Cancel"       => 'print "Cancel\n";',
                     "Help"         => 'print "Help\n";',
                     "ApplyToAll"   => 'print "ApplyToAll\n";',
                     "Disassociate" => 'print "Disassociate\n";',
                     "Exit"        => 'CQCC::IOPtkPanel::ptk_exit();',
                   ],
               PACK=>[-side => "right", -fill => 'y']);
    $IO->RunMainLoop();
    return 0;
}

######################################################################
# method ptk_exit
######################################################################

sub ptk_exit {
    my $i;
    &IOstd()->GetMainWindow()->DESTROY();
}

######################################################################
#::DESTROY

=over 6

=item C<DESTROY>

This internal destructor method in turn destroys the IOPtkWidget
child objects then calls the main destructor.

=back

=cut

######################################################################

sub DESTROY {
    my ($self) = @_;
    my %children = $self->{CHILDREN} ? %{ $self->{CHILDREN} } : ();
    my @childList = values %children;
    for (@childList) { 
        $_->DESTROY();
    } 
    # CCperl doesn't support SUPER so use direct reference to parent
    $self->CQCC::IOPtkWidget::DESTROY();
}

######################################################################
#::DeleteWindowCallback

=over 6

=item C<DeleteWindowCallback>

Invoked when the window system deletes a top level window. 

=back

=cut

######################################################################

sub DeleteWindowCallback {
    # Ideally ought to call TriggerCQCC::Action_Cancel to savecache too
    &IOstd()->ExitHandler(1);
}

######################################################################
#::method IODialogBox

=over 6

=item C<IODialogBox subtype>

Calling sequence using an instance of IOPerlTK called $IO:

    $IO->Panel(name, parent, "IODialogBox", key-values)>

This subtype method is called during initialization to define an
IODialogBox panel widget. It uses the Perl/TK DialogBox object
directly after translating its TITLE and button titles through the
message catalog. If a BUTTONS attribute is provided the buttons are
based on that msgtag list reference; otherwise it will use the
standard OK and Cancel button names.

The IODialogBox is hidden until it invoked using the IODialogBoxShow
method described below. 

=back

=cut

######################################################################

sub IODialogBox {
   my ($self) = @_;

   my @labelArgs = $self->{TITLEMSG} ? 
                   ("-title" => $self->{TITLEMSG}) : ();
   my @buttons = $self->{BUTTONS} ? @{ $self->{BUTTONS} } : 
                   ( ["GUI_Commit"], ["GUI_Cancel"]);
   my @buttonMsgs;
   for (@buttons) {
       push @buttonMsgs, $self->{MSGCAT}->Message($_);
   }

   ######################################### Perl/TK code
   $self->{PARENT_WIDGET} = &IOstd()->GetMainWindow();
   $self->{WIDGET} = $self->{PARENT_WIDGET}->
       DialogBox(@labelArgs,
                 -buttons => \@buttonMsgs,
                 $self->TKoptions()); 
   if ($self->{GEOMETRY}) { 
       $self->{WIDGET}->geometry($self->{GEOMETRY});
   }
   ########################################

   return $self;
}

######################################################################
#::method IODialogBoxShow

=over 6

=item C<IODialogBoxShow()>

This method is used to present a IODialogBox subtype panel, wait
for a button selection, then return. If the selection matches
the standard Cancel message, it will return a 1 for abort, otherwise
it will return 0. 

If this method is called for an IOPtkPanel that is not a IODialogBox,
it will do nothing and return a 0. 

=back

=cut

#####################################################################

sub IODialogBoxShow {
    my ($self) = @_;
    my $result = 0;
    if ($self->{TYPE} eq "IODialogBox") {
         my $answer = $self->{WIDGET}->Show();
         my $abortMsg = $self->{MSGCAT}->Message(["GUI_Cancel"]);
         if ($answer eq $abortMsg) {
             $result = 1;
         }
    }
    return $result;
}

######################################################################
#::method IOFrame

=over 6

=item C<IOFrame subtype>

Calling sequence using an instance of IOPerlTK called $IO:

    $IO->Panel(name, parent, "IOFrame", key-values)>

This subtype method is called during initialization to define a 
generic Frame panel widget. It uses the Perl/TK Frame object 
directly after translating its TITLE.

=back

=cut

######################################################################

sub IOFrame {
   my ($self) = @_;

   ######################################### Perl/TK code
   my @packArgs = $self->PackArgs( -fill => "both", -expand => 1 );
   my @labelArgs = $self->{TITLEMSG} ? 
                   ("-label" => $self->{TITLEMSG}) : ();
   $self->{WIDGET} = $self->{PARENT_WIDGET}->
       Frame(@labelArgs, $self->TKoptions()
             )->pack(@packArgs);
   ########################################

   return $self;
}

######################################################################
#::method ManageChildren

=over 6

=item C<ManageChildren(operation, child)>

This method can be called for any IOPtkPanel widget to modify
its child list. Operations currently supported include the following:
ADD, which adds a child to the list and DELETE which deletes the 
specified list. With no arguments, it will return a sorted list 
of child names. 

Currently, this method is only called by IOPtkWidget as part of
the generic services it provides.

=back

=cut

######################################################################

sub ManageChildren {
   my ($self,$op,$child) = @_;
   my %children = $self->{CHILDREN} ? %{ $self->{CHILDREN} } : ();
   
   if ($op eq "ADD") { 
      $children{$child->{NAME}} = $child;
      $self->{CHILDREN} = \%children;

   } elsif ($op eq "DELETE") { 
      delete $children{$child->{NAME}};
      $self->{CHILDREN} = \%children;
   }    
   return sort keys %children;
}

######################################################################
#::method new

=over 6

=item C<new CQCC::IOPtkPanel()>

This method builds on the IOPtkWidget constructor to add an empty 
child list and then calls the subtype initialization method. 

The initializers for IOMainWindow and IOTopLevel are built into this
method directly. 

=back

=cut

######################################################################

sub new { 
   my $proto = shift;
   my $class = ref($proto) || $proto;  # Object or class name

   # Attributes
   my $self = new CQCC::IOPtkWidget(@_);

   $self->{CLASSNAME} = "CQCC::IOPtkPanel";
   $self->{CHILDREN} = undef;

   bless($self, $class);

   # Process panel subtypes for Perl/TK initialization

   my $type = $self->{TYPE};
   if ($type eq "IOMainWindow") { 
      ######################################### Perl/TK code
      # Delete the TempMain window if it was created already
      if ($self->{NAME} ne "TempMain") { 
          my $tempMain = WidgetLookup("TempMain", 1);
          if ($tempMain) {
               $tempMain->{WIDGET}->DESTROY();
               $tempMain->DESTROY();
          }
      } else {
          $self->{GEOMETRY} = "1x1";
      }
      $self->{WIDGET} = Tk::MainWindow->new($self->TKoptions());
      if ($self->{TITLEMSG}) {
          $self->{WIDGET}->title($self->{TITLEMSG});
      }
      if ($self->{GEOMETRY}) {
          $self->{WIDGET}->geometry($self->{GEOMETRY});
      }
      if ($] ge "5.008006" && $Config::Config{osname} ne "MSWin32") {
          $self->{WIDGET}->useinputmethods(1);
      }
          
      &IOstd()->SetMainWindow($self->{WIDGET});
      $self->{WIDGET}->protocol( "WM_DELETE_WINDOW", \&CQCC::IOPtkPanel::DeleteWindowCallback);

      #########################################

   } elsif ($type eq "IOTopLevel") {
      ######################################### Perl/TK code
      $self->{WIDGET} = &IOstd()->GetMainWindow()->
          Toplevel($self->TKoptions());
      if ($self->{TITLEMSG}) {
          $self->{WIDGET}->title($self->{TITLEMSG});
      }
      if ($self->{GEOMETRY}) {
          $self->{WIDGET}->geometry($self->{GEOMETRY});
      }
      $self->{WIDGET}->raise();
      ########################################

   } else {
      $self->NewSubType();
   }
   return $self;
}

######################################################################
#::Tail
######################################################################

1;

